package controllers

import (
	"crypto/tls"
	"devops/dto"
	"devops/utils"
	"encoding/json"
	"errors"
	"github.com/astaxie/beego"
	"io/ioutil"
	"net/http"
	"time"
)

// Operations about Credential
type CredentialController struct {
	beego.Controller
}

// URLMapping ...
func (c *CredentialController) URLMapping() {
	c.Mapping("getAllCredentials", c.GetAllCredentials)
}

// getAllCredentials ...
// @Title getAllCredentials
// @Description get AllCredentials
// @Param	lang		query 	string	true	"api result language type"
// @Success 200 {object} dto.BaseResultResponseDto
// @Failure 300 {object} dto.BaseResponseDto
// @router /getAllCredentials [post]
func (c *CredentialController) GetAllCredentials() {
	lang := c.GetString("lang")
	infoLogger.Println("语言: ", lang)
	var errorResponseDto = dto.BaseResponseDto{300, utils.GetI18nLang(lang, "sysError")}
	var successResponseDto = dto.BaseResultResponseDto{}
	successResponseDto.Status = 200

	// 生成ApiToken
	//apiToken := generateApiToken()
	//apiToken := getApiToken()

	//infoLogger.Printf("调用生成ApiToken接口返回值:%s", apiToken)

	const BaseUrl = "http://jenkins.poc.paas.com"
	httpClient := &http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		},
		Timeout: 10 * time.Second,
		CheckRedirect: func(req *http.Request, via []*http.Request) error {
			return http.ErrUseLastResponse
		},
	}

	getAllJobUrl := BaseUrl + "/blue/rest/organizations/jenkins/pipelines/"
	getAllJobReq, _ := http.NewRequest("GET", getAllJobUrl, nil)

	getAllJobRes, err := httpClient.Do(getAllJobReq)
	if err != nil {
		errorLogger.Println("调用获取所有job接口失败:", err.Error())
		errorResponseDto.Message = "获取所有job失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	getAllJobResByte, err := ioutil.ReadAll(getAllJobRes.Body)
	if err != nil {
		errorLogger.Printf("解析调用获取所有job接口返回参数失败:%s", err.Error())
		errorResponseDto.Message = "解析调用获取所有job接口返回参数失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	var resMap []map[string]interface{}
	if err := json.Unmarshal(getAllJobResByte, &resMap); err != nil {
		errorLogger.Println("解析获取所有job接口返回数据失败:", err.Error())
		errorResponseDto.Message = "解析获取所有job接口返回数据失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	allJobCount := len(resMap)
	infoLogger.Println("all job:", allJobCount)
	jobName := "temp-job"
	if allJobCount == 0 {
		//执行创建临时job
		execCreatePipelineJobRes := ExecCreatePipelineJob(jobName)
		if execCreatePipelineJobRes.Status == 300 {
			errorLogger.Printf("执行创建临时job:%s失败:%s", jobName, execCreatePipelineJobRes.Message)
			errorResponseDto.Message = execCreatePipelineJobRes.Message
			c.Data["json"] = errorResponseDto
			c.ServeJSON()
			return
		}
	} else {
		jobName = resMap[0]["fullName"].(string)
	}
	infoLogger.Println("job name:", jobName)

	// 获取所有凭据
	allCredentials, err := GetCredentials(jobName)
	if err != nil {
		errorLogger.Println("调用获取所有凭据接口失败:", err.Error())
		errorResponseDto.Message = "获取所有凭据接口失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	if len(allCredentials) > 0 {
		successResponseDto.Result = allCredentials
		c.Data["json"] = successResponseDto
		c.ServeJSON()
		return
	}

	defaultCredentialName := beego.AppConfig.String("defaultCredentialName")
	//执行创建凭据 默认名称k2admin 密码K2pass!!
	createCredentialsParams := "_.scope=GLOBAL&_.username=k2admin&_.password=K2pass%21%21&_.id=&_.description=" +
		"&stapler-class=com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl&%24class=" +
		"com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl&stapler-class=" +
		"org.jenkinsci.plugins.docker.commons.credentials.DockerServerCredentials&%24class=" +
		"org.jenkinsci.plugins.docker.commons.credentials.DockerServerCredentials&stapler-class=" +
		"com.dabsquared.gitlabjenkins.connection.GitLabApiTokenImpl&%24class=" +
		"com.dabsquared.gitlabjenkins.connection.GitLabApiTokenImpl&stapler-class=" +
		"org.jenkinsci.plugins.kubernetes.credentials.FileSystemServiceAccountCredential&%24class=" +
		"org.jenkinsci.plugins.kubernetes.credentials.FileSystemServiceAccountCredential&stapler-class=" +
		"org.jenkinsci.plugins.kubernetes.credentials.OpenShiftBearerTokenCredentialImpl&%24class=" +
		"org.jenkinsci.plugins.kubernetes.credentials.OpenShiftBearerTokenCredentialImpl&stapler-class=" +
		"com.cloudbees.jenkins.plugins.sshcredentials.impl.BasicSSHUserPrivateKey&%24class=" +
		"com.cloudbees.jenkins.plugins.sshcredentials.impl.BasicSSHUserPrivateKey&stapler-class=" +
		"org.jenkinsci.plugins.plaincredentials.impl.FileCredentialsImpl&%24class=" +
		"org.jenkinsci.plugins.plaincredentials.impl.FileCredentialsImpl&stapler-class=" +
		"org.jenkinsci.plugins.plaincredentials.impl.StringCredentialsImpl&%24class=" +
		"org.jenkinsci.plugins.plaincredentials.impl.StringCredentialsImpl&stapler-class=" +
		"com.cloudbees.plugins.credentials.impl.CertificateCredentialsImpl&%24class=" +
		"com.cloudbees.plugins.credentials.impl.CertificateCredentialsImpl&json=" +
		"%7B%22%22%3A+%220%22%2C+%22credentials%22%3A+%7B%22scope%22%3A+%22GLOBAL%22%2C+%22username%22%3A+%22" +
		defaultCredentialName + "%22%2C+%22password%22%3A+%22K2pass%21%21%22%2C+%22%24redact%22%3A+%22password%22%2C+" +
		"%22id%22%3A+%22%22%2C+%22description%22%3A+%22%22%2C+%22stapler-class%22%3A+%22" +
		"com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl%22%2C+%22%24class%22%3A+%22" +
		"com.cloudbees.plugins.credentials.impl.UsernamePasswordCredentialsImpl%22%7D%7D&Submit=%E7%A1%AE%E5%AE%9A"
	getAllCredentialsUrl := BaseUrl + "/credentials/store/system/domain/_/createCredentials?" + createCredentialsParams

	getAllCredentialsReq, _ := http.NewRequest("POST", getAllCredentialsUrl, nil)
	getAllCredentialsReq.Header.Set("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8")

	getAllCredentialsRes, err := httpClient.Do(getAllCredentialsReq)
	if err != nil {
		errorLogger.Printf("执行创建默认凭据:%s失败:%s", defaultCredentialName, err.Error())
		errorResponseDto.Message = "执行创建默认凭据失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	infoLogger.Println(getAllCredentialsRes)

	infoLogger.Printf("执行创建默认凭据:%s返回状态码:%s", defaultCredentialName, getAllCredentialsRes.Status)
	if getAllCredentialsRes.StatusCode != http.StatusFound {
		errorResponseDto.Message = "执行创建默认凭据失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}
	infoLogger.Printf("执行创建默认凭据%s成功", defaultCredentialName)

	//if jobName == "temp-job"{
	//	deleteUrl := BaseUrl + "/job/" + jobName + "/doDelete"
	//	req, _ := http.NewRequest("POST", deleteUrl, nil)
	//	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")
	//
	//	deleteRes, err := httpClient.Do(req)
	//	if err != nil {
	//		errorLogger.Printf("执行删除临时job:%s失败:%s", jobName, err.Error())
	//		errorResponseDto.Message = "执行删除临时job失败"
	//		c.Data["json"] = errorResponseDto
	//		c.ServeJSON()
	//		return
	//	}
	//	infoLogger.Printf("执行删除临时job:%s返回状态码:%s", jobName, deleteRes.Status)
	//	if deleteRes.StatusCode != http.StatusFound {
	//		errorResponseDto.Message = "执行删除临时job失败"
	//		c.Data["json"] = errorResponseDto
	//		c.ServeJSON()
	//		return
	//	}
	//	infoLogger.Printf("执行删除job:%s成功", jobName)
	//}

	// 获取所有凭据
	allCredentials, err = GetCredentials(jobName)
	if err != nil {
		errorLogger.Println("调用获取所有凭据接口失败:", err.Error())
		errorResponseDto.Message = "获取所有凭据接口失败"
		c.Data["json"] = errorResponseDto
		c.ServeJSON()
		return
	}

	successResponseDto.Result = allCredentials
	c.Data["json"] = successResponseDto
	c.ServeJSON()
}

// 获取所有凭据
func GetCredentials(jobName string) ([]dto.GetAllCredentialsResultResponseDto, error) {

	result := []dto.GetAllCredentialsResultResponseDto{}

	const BaseURL = "http://jenkins.poc.paas.com"
	httpClient := &http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		},
		Timeout: 10 * time.Second,
		CheckRedirect: func(req *http.Request, via []*http.Request) error {
			return http.ErrUseLastResponse
		},
	}

	getAllCredentialsParams := "url=&credentialsId="
	getAllCredentialsUrl := BaseURL + "/job/" + jobName +
		"/descriptorByName/hudson.plugins.git.UserRemoteConfig/fillCredentialsIdItems?" + getAllCredentialsParams

	getAllCredentialsReq, _ := http.NewRequest("POST", getAllCredentialsUrl, nil)
	getAllCredentialsReq.Header.Set("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8")

	getAllCredentialsRes, err := httpClient.Do(getAllCredentialsReq)
	if err != nil {
		errorLogger.Println("调用获取所有凭据接口失败:", err.Error())
		return result, errors.New("调用获取所有凭据接口失败")
	}

	getAllCredentialsResByte, err := ioutil.ReadAll(getAllCredentialsRes.Body)
	if err != nil {
		errorLogger.Printf("解析调用获取所有凭据接口返回参数失败:%s", err.Error())
		return result, errors.New("解析调用获取所有凭据接口返回参数失败")
	}

	var getAllCredentialsResMap map[string]interface{}
	if err := json.Unmarshal(getAllCredentialsResByte, &getAllCredentialsResMap); err != nil {
		errorLogger.Println("解析获取获取所有凭据接口返回数据失败:", err.Error())
		return result, errors.New("解析调用获取所有凭据接口返回参数失败")
	}

	allCredentials := getAllCredentialsResMap["values"].([]interface{})
	if len(allCredentials) > 1 {
		allCredentials = allCredentials[1:]
		for _, c := range allCredentials {
			credential := c.(map[string]interface{})
			name := credential["name"].(string)
			infoLogger.Println("credential name:", name)
			value := credential["value"].(string)
			infoLogger.Println("credential value:", value)

			result = append(result, dto.GetAllCredentialsResultResponseDto{
				value,
				name,
			})
		}
	}

	return result, nil
}
