package main

import (
	_ "devops/routers"
	"github.com/astaxie/beego/context"
	"github.com/astaxie/beego/logs"
	"github.com/astaxie/beego/plugins/cors"
	"strings"

	"github.com/astaxie/beego"
)

func main() {
	if beego.BConfig.RunMode == "dev" {
		beego.BConfig.WebConfig.DirectoryIndex = true
		beego.BConfig.WebConfig.StaticDir["/swagger"] = "swagger"
	}

	//日志
	beego.SetLogFuncCall(true)
	beego.SetLevel(beego.LevelInformational)
	beego.SetLogger("file", `{"filename":"logs/devops.log", "daliy":false, "maxdays": 30}`)

	//设置静态页面
	beego.SetStaticPath("/", "/go/frontend/dist/index.html")
	beego.BConfig.WebConfig.StaticDir["/favicon.ico"] = "/go/frontend/dist/favicon.ico"
	beego.BConfig.WebConfig.StaticDir["/static"] = "/go/frontend/dist/static"
	beego.BConfig.WebConfig.StaticDir["/static/*"] = "/go/frontend/dist/static/*"

	//跨域设置
	beego.InsertFilter("*", beego.BeforeRouter, cors.Allow(&cors.Options{
		AllowAllOrigins: true,
		AllowMethods:    []string{"GET", "POST", "PUT", "DELETE", "OPTIONS"},
		AllowHeaders: []string{"Origin", "Authorization", "Access-Control-Allow-Origin",
			"Access-Control-Allow-Headers", "Content-Type", "x-token"},
		ExposeHeaders:    []string{"Content-Length", "Access-Control-Allow-Origin", "Access-Control-Allow-Headers", "Content-Type"},
		AllowCredentials: true,
	}))
	beego.InsertFilter("*", beego.AfterExec, beego.FilterFunc(func(c *context.Context) {
		c.Output.Header("Access-Control-Allow-Origin", "*")
	}))

	//语言参数值校验
	beego.InsertFilter("*", beego.BeforeExec, beego.FilterFunc(func(c *context.Context) {
		c.Output.Header("Access-Control-Allow-Origin", "*")
		query := c.Request.URL.Query().Encode()
		if strings.Contains(query, "lang") {
			lang := c.Request.URL.Query().Get("lang")
			logs.GetLogger().Println("语言: ", lang)
			if lang != "" {
				if lang != "zh" && lang != "en" {
					m := map[string]interface{}{"status": 200, "message": "语言参数值不正确"}
					c.Output.JSON(m, false, false)
					return
				}
			} else {
				c.Request.URL.Query().Set("lang", "zh")
			}
		}
	}))

	beego.Run()
}
