package utils

import (
	"flag"
	"github.com/astaxie/beego"
	"github.com/astaxie/beego/logs"
	"k8s.io/client-go/kubernetes"
	_ "k8s.io/client-go/plugin/pkg/client/auth/oidc"
	"k8s.io/client-go/tools/clientcmd"
)

var (
	nameSpace   = beego.AppConfig.String("nameSpace")
	kubeconfig  = flag.String("kubeconfig", beego.AppConfig.String("kubeConfig"), "kubeconfig file path")
	errorLogger = logs.GetLogger("error")
)

// 获取k8s客户端
func GetK8sClient() *kubernetes.Clientset {

	flag.Parse()

	//解析config文件
	config, err := clientcmd.BuildConfigFromFlags("", *kubeconfig)
	if err != nil {
		//panic(err.Error())
		errorLogger.Println(err.Error())
		return nil
	}

	//创建k8s clientset
	clientset, err := kubernetes.NewForConfig(config)
	if err != nil {
		//panic(err.Error())
		errorLogger.Println(err.Error())
		return nil
	}
	return clientset
}
